' Batch for shocking exogenous variables when considering both supply and demand side of the housing market. Replicates figures in Section 7.2 of the paper
' written by Andr Anundsen
' Date 21.09.2011

close @all

%path = @runpath
cd %path

%START = "1970Q1"	'First observation
%END =  "2035q4"         'End of sample
%STOP = "2008q4"  	' last period in estimation period
%FSTART = "2009q1"   ' first solution period in forecast/simulation
%FSTOP  = "2016q4"    'last solution period in forecast/simulation


' Creating a workfile

wfcreate model q %start %end

' Creating centered seasonal dummies 

SERIES CS1 = @SEAS(1)-.25
SERIES CS2 = @SEAS(2)-.25
SERIES CS3 = @SEAS(3)-.25

' Open a database from PcGive (in7 file)

dbopen "H:\Prosjekter\Boligpriser i Norge\Replication files\Data\Data.in7"  

'Extracting the relevant data.  

fetch pbokap rd300 ram300 kpi pc rd300 renpf300 k83 omsetning f bg300 trtmnw xsbolu pjks83 jks83 

smpl @all

'Creating dummies for supply side model (From KVARTS)

series d02q2= @recode(@date>@dateval("2002/1") and @date<@dateval("2002/3"),1,0)
series d86q2 = @recode(@date>@dateval("1986/1") and @date<@dateval("1990/1"),1,0)
series d86q3 = @recode(@date>@dateval("1986/2") and @date<@dateval("1990/1"),1,0)
    
'*******************************************************************
' Formulating the system
'******************************************************************* 

smpl 1980q1 @LAST        

'Start by defining the exogenous variables in the system 

series ydp = log((rd300-ram300)/pc)     						'Real disposable income
series r = ((1+4*renpf300*(1-trtmnw)) /(kpi/kpi(-4)))-1		'Real interest rate after tax
series t = log(omsetning)											'Turnover
series q = f/100														'Households expectations


'Creating variables to shock Dph and Dd 

'Residual in housing price equation

series e1 = 0

'Residual in credit equation

series e2 = 0

'Defining the system

smpl 1986q2 @last

' Below, we formulate the housing-credit system

system sys_phandd

'Housing price equation

sys_phandd.append Dlog(pbokap/pc) =  C(198)*( log(pbokap(-1)/pc(-1))-0.97821*log(bg300(-1)/pc(-1))-1.6907*ydp(-1)+3.0267*log(k83(-1))) + C(199)*( log(bg300(-1)/pc(-1))-0.75903*log(pbokap(-1)/pc(-1))+2.7419*r(-2)-0.28070*t(-1)-0.75903*log(k83(-1)))+ C(111)*Dlog(bg300/pc)+C(112)*Dlog(bg300(-3)/pc(-3))+C(113)*Dlog(pbokap(-4)/pc(-4))+C(114)*D(q)+C(115)*D(q(-1))+C(116)*D(q(-2))+C(117)*CS1+C(118)*CS2+C(119)*CS3+C(120)

'Credit equation

sys_phandd.append Dlog(bg300/pc)=  C(299)*( log(bg300(-1)/pc(-1))-0.75903*log(pbokap(-1)/pc(-1))+2.7419*r(-2)-0.28070*t(-1)-0.75903*log(k83(-1)))+ C(211)*Dlog(bg300(-1)/pc(-1))+C(212)*D(ydp(-3))+ C(213)*D(r(-4))+C(214)*CS1+C(215)*CS2+C(216)*CS3+C(217)

'Estimating the system by FIML

sys_phandd.FIML(deriv=n)

'Defining supply side

'Housing starts (estimated by OLS)

equation eq_igang.ls Dlog(xsbolu) = C(1)+C(2)*dlog(xsbolu(-4))+C(3)*(log(xsbolu(-1))-log(pbokap(-1)/pjks83(-1)))+C(4)*@seas(1)+C(5)*d02q2+C(6)*d86q2*@seas(2)+C(7)*d86q3*@seas(3)

'Investments (estimated by OLS) 

equation eq_inv.ls Dlog(jks83)  =  Dlog(0.3124*xsbolu+0.2455*xsbolu(-1)+0.1672*xsbolu(-2)+0.1125*xsbolu(-3)+0.0702*xsbolu(-4)+0.0407*xsbolu(-5)+0.0235*xsbolu(-6)+0.0131*xsbolu(-7)+0.0074*xsbolu(-8)+0.0043*xsbolu(-9)+0.0021*xsbolu(-10)+0.0009*xsbolu(-11)+0.0002*xsbolu(-12))+ C(500)*cs1+C(501)*cs2+C(502)*cs3


'Creating a model to be used to study dynamic multipliers (merging supply side to housing price and credit system)

sys_phandd.makemodel(model)
model.merge eq_igang
model.merge eq_inv

'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'Creating a "scenario" code to look at dynamic multipliers

'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' First, add identities to the model to look at dynamic responses
 
smpl @all

'Growth in housing prices

series Dph = log(pbokap/pc)-log(pbokap(-1)/pc(-1))
model.append @identity Dph =log(pbokap/pc)-log(pbokap(-1)/pc(-1))

'Credit growth 

series Dd = log(bg300/pc)-log(bg300(-1)/pc(-1))
model.append @identity  Dd =log(bg300/pc)-log(bg300(-1)/pc(-1))

'Housing prices

series ph = log(pbokap/pc)
model.append @identity ph=log(pbokap/pc)

'Household debt

series debt = log(bg300/pc)
model.append @identity debt=log(bg300/pc)

'Investments

model.append @identity inv = log(jks83)

'Housing stock (law of motion equation)

model.append @identity k83 = (1-0.004)*k83(-1)+jks83


' Creating the override variable (here we use e1 for housing price shock, e2 for credit shock and ydp for shock in income). Currently we look at shock in income, but can easily be changed by shifting out ydp with e1 or e2 below

series ydp_ydp = ydp
smpl 1986q2 2008q4

' Increase income by 1 in 1987:1 (here the other scenarios can also easily be considered by changing the below code)

ydp_ydp =ydp+0.01	
smpl 1986q2 2008q4		

'Creating scenario with _ydp
	
'model is just the name of the model. 

'n refers to a new scenario, while a is the aliased variable

model.scenario(n, a=ydp)   "ydp_ydp"

' Substitute in override variable

'Model just refers to the model, which I have called model. The override function tells the program to override the existing ydp variable. Could list more variables to create more comlicated scenarios. 

model.override ydp

' Compare to baseline scenario (the model as it is)

model.scenario(c) "baseline"

'm is the number of iterations, s is solution type (a means that we collect se, conficence bounds and means), d=d --> dynamic solution, a --> can solve both alternate and active scenario, o = solution method (g is Gauss), b is size of confidence intervals, cu = coefficient uncertainty (t=true)

model.solveopt(M=5000, S=A, D=D, R=1000, B=0.95, A=T, O=G, CU=t)
model.solve

' Making dynamic multipliers to see how different variables respond

model.makegraph(N, D, S=B, G=V)  graph_rup  Dph Dd  k83 ph debt inv 
graph_rup.axis(l) grid
graph_rup.align(3,1,1) 
show graph_rup
smpl @all


